Module: MiGA

Included in:
MiGA
Defined in:
lib/miga/version.rb

Overview

High-level minimal requirements for the MiGA::MiGA class.

Defined Under Namespace

Modules: Common, TaxDist Classes: Cli, Daemon, Dataset, Json, Metadata, MiGA, Project, RemoteDataset, Result, TaxIndex, TaxIndexTaxon, Taxonomy

Constant Summary collapse

VERSION =

Current version of MiGA. An Array with three values:

  • Float representing the major.minor version.

  • Integer representing gem releases of the current version.

  • Integer representing minor changes that require new version number.

[0.6, 3, 2]
VERSION_NAME =

Nickname for the current major.minor version.

'lithograph'
VERSION_DATE =

Date of the current gem release.

Date.new(2020, 4, 13)
CITATION =

Reference of MiGA.

'Rodriguez-R et al (2018). ' \
'The Microbial Genomes Atlas (MiGA) webserver: taxonomic and gene ' \
'diversity analysis of Archaea and Bacteria at the whole genome level. ' \
'Nucleic Acids Research 46(W1):W282-W288. doi:10.1093/nar/gky467.'