Class: MiGA::RemoteDataset

Inherits:
MiGA
  • Object
show all
Includes:
Base, Download
Defined in:
lib/miga/remote_dataset.rb,
lib/miga/remote_dataset/base.rb,
lib/miga/remote_dataset/download.rb

Overview

MiGA representation of datasets with data in remote locations.

Defined Under Namespace

Modules: Base, Download

Constant Summary

Constants included from MiGA

CITATION, VERSION, VERSION_DATE, VERSION_NAME

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Download

#download

Methods inherited from MiGA

CITATION, DEBUG, DEBUG_OFF, DEBUG_ON, DEBUG_TRACE_OFF, DEBUG_TRACE_ON, FULL_VERSION, LONG_VERSION, VERSION, VERSION_DATE, initialized?, #result_files_exist?

Methods included from Common::Path

#root_path, #script_path

Methods included from Common::Format

#clean_fasta_file, #seqs_length, #tabulate

Constructor Details

#initialize(ids, db, universe) ⇒ RemoteDataset

Initialize MiGA::RemoteDataset with ids in database db from universe.



40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
# File 'lib/miga/remote_dataset.rb', line 40

def initialize(ids, db, universe)
  ids = [ids] unless ids.is_a? Array
  @ids = (ids.is_a?(Array) ? ids : [ids])
  @db = db.to_sym
  @universe = universe.to_sym
   = {}
  [:"#{universe}_#{db}"] = ids.join(',')
  @@UNIVERSE.keys.include?(@universe) or
    raise "Unknown Universe: #{@universe}. Try: #{@@UNIVERSE.keys}"
  @@UNIVERSE[@universe][:dbs].include?(@db) or
    raise "Unknown Database: #{@db}. Try: #{@@UNIVERSE[@universe][:dbs]}"
  @_ncbi_asm_json_doc = nil
  # FIXME: Part of the +map_to+ support:
  # unless @@UNIVERSE[@universe][:dbs][@db][:map_to].nil?
  #   MiGA::RemoteDataset.download
  # end
end

Instance Attribute Details

#dbObject (readonly)

Database storing the dataset.



30
31
32
# File 'lib/miga/remote_dataset.rb', line 30

def db
  @db
end

#idsObject (readonly)

Array of IDs of the entries composing the dataset.



32
33
34
# File 'lib/miga/remote_dataset.rb', line 32

def ids
  @ids
end

#metadataObject (readonly)

Internal metadata hash



34
35
36
# File 'lib/miga/remote_dataset.rb', line 34

def 
  
end

#universeObject (readonly)

Universe of the dataset.



28
29
30
# File 'lib/miga/remote_dataset.rb', line 28

def universe
  @universe
end

Class Method Details

.download(universe, db, ids, format, file = nil, extra = [], obj = nil) ⇒ Object

Download data from the universe in the database db with IDs ids and in format. If passed, it saves the result in file. Additional parameters specific to the download method can be passed using extra. Returns String. The obj can also be passed as MiGA::RemoteDataset or MiGA::Dataset.



15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
# File 'lib/miga/remote_dataset/download.rb', line 15

def download(universe, db, ids, format, file = nil, extra = [], obj = nil)
  ids = [ids] unless ids.is_a? Array
  getter = @@UNIVERSE[universe][:dbs][db][:getter] || :download
  method = @@UNIVERSE[universe][:method]
  opts = {
    universe: universe,
    db: db,
    ids: ids,
    format: format,
    file: file,
    extra: extra,
    obj: obj
  }
  doc = send("#{getter}_#{method}", opts)
  unless file.nil?
    ofh = File.open(file, 'w')
    ofh.print doc
    ofh.close
  end
  doc
end

.download_rest(opts) ⇒ Object Also known as: download_net

Download data using the REST method. Supported opts (Hash) include: universe (mandatory): Symbol db (mandatory): Symbol ids (mandatory): Array of String format: String extra: Array



59
60
61
62
63
64
65
# File 'lib/miga/remote_dataset/download.rb', line 59

def download_rest(opts)
  u = @@UNIVERSE[opts[:universe]]
  url = sprintf(u[:url],
    opts[:db], opts[:ids].join(','), opts[:format], *opts[:extra])
  url = u[:api_key][url] unless u[:api_key].nil?
  download_url url
end

.download_url(url) ⇒ Object

Download the given url and return the result regardless of response code. Attempts download up to three times before raising Net::ReadTimeout.



74
75
76
77
78
79
80
81
82
83
84
85
86
# File 'lib/miga/remote_dataset/download.rb', line 74

def download_url(url)
  doc = ''
  @timeout_try = 0
  begin
    DEBUG 'GET: ' + url
    open(url, read_timeout: 600) { |f| doc = f.read }
  rescue => e
    @timeout_try += 1
    raise e if @timeout_try >= 3
    retry
  end
  doc
end

.ncbi_asm_acc2id(acc) ⇒ Object



15
16
17
18
19
20
21
# File 'lib/miga/remote_dataset.rb', line 15

def ncbi_asm_acc2id(acc)
  return acc if acc =~ /^\d+$/
  search_doc = MiGA::Json.parse(
    download(:ncbi_search, :assembly, acc, :json),
    symbolize: false, contents: true)
  (search_doc['esearchresult']['idlist'] || []).first
end

.ncbi_asm_rest(opts) ⇒ Object

Download data from NCBI Assembly database using the REST method. Supported opts (Hash) include: obj (mandatory): MiGA::RemoteDataset ids (mandatory): String or Array of String file: String, passed to download extra: Array, passed to download format: String, passed to download



45
46
47
48
49
50
# File 'lib/miga/remote_dataset/download.rb', line 45

def ncbi_asm_rest(opts)
  url_dir = opts[:obj].ncbi_asm_json_doc['ftppath_genbank']
  url = "#{url_dir}/#{File.basename url_dir}_genomic.fna.gz"
  download(:web, :assembly_gz, url,
    opts[:format], opts[:file], opts[:extra], opts[:obj])
end

.ncbi_map(id, dbfrom, db) ⇒ Object

Looks for the entry id in dbfrom, and returns the linked identifier in db (or nil).



91
92
93
94
95
96
97
98
99
100
# File 'lib/miga/remote_dataset/download.rb', line 91

def ncbi_map(id, dbfrom, db)
  doc = download(:ncbi_map, dbfrom, id, :json, nil, [db])
  return if doc.empty?
  tree = MiGA::Json.parse(doc, contents: true)
  [:linksets, 0, :linksetdbs, 0, :links, 0].each do |i|
    tree = tree[i]
    break if tree.nil?
  end
  tree
end

.UNIVERSEObject



9
# File 'lib/miga/remote_dataset/base.rb', line 9

def UNIVERSE ; @@UNIVERSE ; end

Instance Method Details

#get_metadata(metadata_def = {}) ⇒ Object

Get metadata from the remote location.



99
100
101
102
103
104
105
106
107
# File 'lib/miga/remote_dataset.rb', line 99

def ( = {})
  .each { |k,v| [k] = v }
  case universe
  when :ebi, :ncbi, :web
    # Get taxonomy
    [:tax] = get_ncbi_taxonomy
  end
   = get_type_status()
end

#get_ncbi_taxidObject

Get NCBI Taxonomy ID.



111
112
113
114
# File 'lib/miga/remote_dataset.rb', line 111

def get_ncbi_taxid
  origin = (universe == :ncbi and db == :assembly) ? :web : universe
  send("get_ncbi_taxid_from_#{origin}")
end

#get_ncbi_taxonomyObject

Get NCBI taxonomy as MiGA::Taxonomy.



131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
# File 'lib/miga/remote_dataset.rb', line 131

def get_ncbi_taxonomy
  tax_id = get_ncbi_taxid
  return nil if tax_id.nil?
  lineage = {ns: 'ncbi'}
  doc = MiGA::RemoteDataset.download(:ncbi, :taxonomy, tax_id, :xml)
  doc.scan(%r{<Taxon>(.*?)</Taxon>}m).map(&:first).each do |i|
    name = i.scan(%r{<ScientificName>(.*)</ScientificName>}).first.to_a.first
    rank = i.scan(%r{<Rank>(.*)</Rank>}).first.to_a.first
    rank = nil if rank == 'no rank' or rank.empty?
    rank = 'dataset' if lineage.empty? and rank.nil?
    lineage[rank] = name unless rank.nil? or rank.nil?
  end
  MiGA.DEBUG "Got lineage: #{lineage}"
  MiGA::Taxonomy.new(lineage)
end

#get_type_status(metadata) ⇒ Object

Get the type material status and return an (updated) metadata hash.



119
120
121
122
123
124
125
126
127
# File 'lib/miga/remote_dataset.rb', line 119

def get_type_status()
  if [:ncbi_asm]
    get_type_status_ncbi_asm 
  elsif [:ncbi_nuccore]
    get_type_status_ncbi_nuccore 
  else
    
  end
end

#ncbi_asm_json_docObject

Get the JSON document describing an NCBI assembly entry.



149
150
151
152
153
154
155
156
157
158
# File 'lib/miga/remote_dataset.rb', line 149

def ncbi_asm_json_doc
  return @_ncbi_asm_json_doc unless @_ncbi_asm_json_doc.nil?
  [:ncbi_asm] ||= ids.first if universe == :ncbi and db == :assembly
  return nil unless [:ncbi_asm]
  ncbi_asm_id = self.class.ncbi_asm_acc2id [:ncbi_asm]
  doc = MiGA::Json.parse(
    self.class.download(:ncbi_summary, :assembly, ncbi_asm_id, :json),
    symbolize: false, contents: true)
  @_ncbi_asm_json_doc = doc['result'][ doc['result']['uids'].first ]
end

#save_to(project, name = nil, is_ref = true, metadata_def = {}) ⇒ Object

Save dataset to the MiGA::Project project identified with name. is_ref indicates if it should be a reference dataset, and contains metadata_def. If metadata_def includes metadata_only: true, no input data is downloaded.



63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
# File 'lib/miga/remote_dataset.rb', line 63

def save_to(project, name = nil, is_ref = true,  = {})
  name ||= ids.join('_').miga_name
  project = MiGA::Project.new(project) if project.is_a? String
  MiGA::Dataset.exist?(project, name) and
    raise "Dataset #{name} exists in the project, aborting..."
   = ()
  udb = @@UNIVERSE[universe][:dbs][db]
  ["#{universe}_#{db}"] = ids.join(',')
  unless [:metadata_only]
    respond_to?("save_#{udb[:stage]}_to", true) or
      raise "Unexpected error: Unsupported stage #{udb[:stage]} for #{db}."
    send "save_#{udb[:stage]}_to", project, name, udb
  end
  dataset = MiGA::Dataset.new(project, name, is_ref, )
  project.add_dataset(dataset.name)
  unless [:metadata_only]
    result = dataset.add_result(udb[:stage], true, is_clean: true)
    result.nil? and
      raise 'Empty dataset: seed result not added due to incomplete files.'
    result.clean!
    result.save
  end
  dataset
end

#update_metadata(dataset, metadata = {}) ⇒ Object

Updates the MiGA::Dataset dataset with the remotely available metadata, and optionally the Hash metadata.



91
92
93
94
95
# File 'lib/miga/remote_dataset.rb', line 91

def (dataset,  = {})
   = ()
  .each { |k,v| dataset.[k] = v }
  dataset.save
end